// Cheque.cpp -- Cheque Classes Methods
#include <iostream>
#include "Cheque.h"

// Cheque Class
Cheque::Cheque(const std::string &s, long an, double bal)
{
    fullName = s;
    acctNum = an;
    balance = bal;
}

void Cheque::Deposit(double amt)
{
    if (amt < 0)
        std::cout << "Negative deposit not allowed; " << "deposit is cancelled.\n";
    else
        balance += amt;
}

void Cheque::Withdraw(double amt)
{
    balance -= amt;
}

// ChequeBase methods
void ChequeBase::Withdraw(double amt)
{
    if (amt < 0)
        std::cout << "Withdrawal amount must be positive; " << "withdrawal canceled.\n";
    else if (amt <= Balance())
        Cheque::Withdraw(amt);
    else
        std::cout << "Withdrawal amount of $" << amt << " exceeds your balance.\n"
                  << "Withdrawal canceled.\n";
}

void ChequeBase::ViewAcct() const
{
    std::cout << "ChequeBase Client: " << FullName() << std::endl;
    std::cout << "Account Number: " << AcctNum() << std::endl;
    std::cout << "Balance: $" << Balance() << std::endl;
}

// ChequePlus Methods
ChequePlus::ChequePlus(const std::string &s, long an, double bal, double ml, double r) : Cheque(s, an, bal)
{
    maxLoan = ml;
    owesBank = 0.0;
    rate = r;
}

ChequePlus::ChequePlus(const ChequeBase &ba, double ml, double r) : Cheque(ba)
{
    maxLoan = ml;
    owesBank = 0.0;
    rate = r;
}

void ChequePlus::ViewAcct() const
{
    std::cout << "ChequePlus Client: " << FullName() << std::endl;
    std::cout << "Account Number: " << AcctNum() << std::endl;
    std::cout << "Balance: $" << Balance() << std::endl;
    std::cout << "Maximum loan: $" << maxLoan << std::endl;
    std::cout << "Owed to bank: $" << owesBank << std::endl;
    std::cout << "Loan Rate: " << 100 * rate << "%\n";
}

void ChequePlus::Withdraw(double amt)
{
    double bal = Balance();
    if (amt <= bal)
        Cheque::Withdraw(amt);
    else if (amt <= bal + maxLoan - owesBank)
    {
        double advance = amt - bal;
        owesBank += advance * (1.0 + rate);
        std::cout << "Bank advance: $" << advance << std::endl;
        std::cout << "Finance charge: $" << advance * rate << std::endl;
        Deposit(advance);
        Cheque::Withdraw(amt);
    }
    else
        std::cout << "Credit limit exceeded. Transaction cancelled.\n";
}
