// Cheque.h -- Cheque classes
#ifndef CHEQUE_H_
#define CHEQUE_H_

#include <iostream>
#include <string>

// Cheque Class
class Cheque
{
private:
    std::string fullName;
    long acctNum;
    double balance;

protected:
    const std::string &FullName() const { return fullName; }
    long AcctNum() const { return acctNum; }

public:
    Cheque(const std::string &s = "Nullbody", long an = -1,
           double bal = 0.0);
    void Deposit(double amt);
    virtual void Withdraw(double amt) = 0; // pure virtual function
    virtual void ViewAcct() const = 0; // pure virtual function
    double Balance() const { return balance; }
    virtual ~Cheque() {}
};

// ChequeBase Class
class ChequeBase : public Cheque
{
public:
    ChequeBase(const std::string &s = "Nullbody", long an = -1,
               double bal = 0.0) : Cheque(s, an, bal) {}
    virtual void Withdraw(double amt);
    virtual void ViewAcct() const;
    virtual ~ChequeBase() {}
};

// ChequePlus Class
class ChequePlus : public Cheque
{
private:
    double maxLoan;
    double rate;
    double owesBank;

public:
    ChequePlus(const std::string &s = "Nullbody", long an = -1,
               double bal = 0.0, double ml = 500,
               double r = 0.10);
    ChequePlus(const ChequeBase &ba, double ml = 500, double r = 0.1);
    virtual void ViewAcct() const;
    virtual void Withdraw(double amt);
    void ResetMax(double m) { maxLoan = m; }
    void ResetRate(double r) { rate = r; }
    void ResetOwes() { owesBank = 0; }
};

#endif