#ifndef SRM_VECTOR_H_
#define SRM_VECTOR_H_
#include <iostream>

template <typename ElemType>
class Vector
{

private:
    ElemType *data;
    int size;
    int capacity;

public:
    Vector();
    Vector(const Vector &vec);
    Vector(ElemType *arr, int num);
    ~Vector();

    Vector &operator=(const Vector &vec);
    ElemType &operator[](int index) const;
    Vector operator+(const Vector &vec);
    Vector &operator++();
    const Vector operator++(int);

    int get_size() const { return size; };
    int get_capacity() const { return capacity; };
    bool empty() const { return size == 0; };

    ElemType at(int index) const;
    ElemType pop_back();
    void push_back(ElemType elem);
    void reserve(int new_capc);
    void shrink_to_fit();
    void insert(int pos, int num = 1, ElemType elem = ElemType{});
    void erase(int first, int last);
    void erase(int index);
    static void swap(Vector &v1, Vector &v2);
};

#endif // SRM_VECTOR_H_


template <typename ElemType>
Vector<ElemType>::Vector() : data(new ElemType[2]), size(0), capacity(2) {}

template <typename ElemType>
Vector<ElemType>::Vector(const Vector &vec) : data(new int[vec.size]), size(vec.size), capacity(vec.size)
{
    for (int i = 0; i < size; i++)
        data[i] = vec.data[i];
}

template <typename ElemType>
Vector<ElemType>::Vector(ElemType *arr, int num) : data(new int[num]), size(num), capacity(num)
{
    for (int i = 0; i < num; i++)
        data[i] = arr[i];
}

template <typename ElemType>
Vector<ElemType>::~Vector()
{
    delete[] data;
    data = nullptr;
    size = capacity = 0;
}

template <typename ElemType>
Vector<ElemType> &Vector<ElemType>::operator=(const Vector<ElemType> &vec)
{
    if (this != &vec)
    {
        size = vec.size;
        capacity = vec.capacity;
        data = new int[capacity];
        for (int i = 0; i < size; i++)
            data[i] = vec.data[i];
    }
    return *this;
}

template <typename ElemType>
ElemType &Vector<ElemType>::operator[](int index) const
{
    if (index < 0 || index >= size)
    {
        std::cout << "Index out of range!" << std::endl;
        exit(0);
    }
    return data[index];
}

template <typename ElemType>
Vector<ElemType> Vector<ElemType>::operator+(const Vector &vec)
{
    if (this->size != vec.size)
    {
        std::cout << "Vectors to be added together must be of the same size!" << std::endl;
        exit(0);
    }
    for (int i = 0; i < size; i++)
        data[i] += vec.data[i];
    return *this;
}

template <typename ElemType>
Vector<ElemType> &Vector<ElemType>::operator++()
{
    for (int i = 0; i < size; i++)
        data[i]++;
    return *this;
}

template <typename ElemType>
const Vector<ElemType> Vector<ElemType>::operator++(int)
{
    Vector old = *this;
    for (int i = 0; i < size; i++)
        data[i]++;
    return old;
}

template <typename ElemType>
ElemType Vector<ElemType>::at(int index) const
{
    if (index < 0 || index >= size)
    {
        std::cout << "Index out of range!" << std::endl;
        exit(0);
    }
    return data[index];
}

template <typename ElemType>
void Vector<ElemType>::push_back(ElemType elem)
{
    if (size == capacity)
    {
        capacity *= 2;
        ElemType *tmp = new ElemType[capacity];
        for (int i = 0; i < size; i++)
            tmp[i] = data[i];
        delete[] data;
        data = tmp;
    }
    data[size++] = elem;
}

template <typename ElemType>
ElemType Vector<ElemType>::pop_back()
{
    if (empty())
    {
        std::cout << "Vector's empty!" << std::endl;
        exit(0);
    }
    return data[--size];
}

template <typename ElemType>
void Vector<ElemType>::reserve(int new_capc)
{
    if (new_capc > capacity)
    {
        ElemType *tmp = new ElemType[new_capc];
        for (int i = 0; i < size; i++)
            tmp[i] = data[i];
        delete[] data;
        data = tmp;
        capacity = new_capc;
    }
}

template <typename ElemType>
void Vector<ElemType>::shrink_to_fit()
{
    if (size < capacity)
    {
        ElemType *tmp = new ElemType[size];
        for (int i = 0; i < size; i++)
            tmp[i] = data[i];
        delete[] data;
        data = tmp;
        capacity = size;
    }
}

template <typename ElemType>
void Vector<ElemType>::insert(int pos, int num, ElemType elem)
{
    if (pos <= 0 || pos > size)
    {
        std::cout << "Invaild position to insert" << std::endl;
        exit(0);
    }
    if (num <= 0)
    {
        std::cout << "Number should be positive" << std::endl;
        exit(0);
    }
    if (size + num > capacity)
        reserve(size + num);
    for (int i = size - 1; i >= pos - 1; i--)
        data[i + num] = data[i];
    for (int i = pos - 1; i < pos + num - 1; i++)
        data[i] = elem;
    size += num;
}

template <typename ElemType>
void Vector<ElemType>::erase(int first, int last)
{
    if (empty())
    {
        std::cout << "Vector's empty" << std::endl;
        exit(0);
    }

    if (first > last || first < 1 || last > size)
    {
        std::cout << "Invalid Range" << std::endl;
        exit(0);
    }
    int new_size = size - (last - first + 1);
    first--;
    for (int i = last; i < size; ++i)
        data[first++] = data[i];
    size = new_size;
}

template <typename ElemType>
void Vector<ElemType>::erase(int index)
{
    if (empty())
    {
        std::cout << "Vector's empty" << std::endl;
        exit(0);
    }
    if (index < 1 || index > size)
    {
        std::cout << "Invalid Range" << std::endl;
        exit(0);
    }
    for (int i = index - 1; i < size; i++)
        data[i] = data[i + 1];
    size--;
}

template <typename ElemType>
void Vector<ElemType>::swap(Vector &v1, Vector &v2)
{
    std::swap(v1.data, v2.data);
    std::swap(v1.size, v2.size);
    std::swap(v1.capacity, v2.capacity);
}
